@@ -11,7 +11,6 @@ import (
 	goflag "flag"
 	"fmt"
 	"io"
-	"io/ioutil"
 	"net/http"
 	"net/url"
 	"os"
@@ -38,7 +37,7 @@ import (
 	"github.com/bitnami-labs/sealed-secrets/pkg/crypto"
 	"github.com/bitnami-labs/sealed-secrets/pkg/multidocyaml"
 
-	ssv1alpha1 "github.com/bitnami-labs/sealed-secrets/pkg/apis/sealed-secrets/v1alpha1"
+	ssv1alpha1 "github.com/bitnami-labs/sealed-secrets/pkg/apis/sealedsecrets/v1alpha1"
 
 	// Register Auth providers
 	_ "k8s.io/client-go/plugin/pkg/client/auth"
@@ -53,53 +52,103 @@ const (
 )
 
 var (
-	// TODO: Verify k8s server signature against cert in kube client config.
-	certURL        = flag.String("cert", "", "Certificate / public key file/URL to use for encryption. Overrides --controller-*")
-	controllerNs   = flag.String("controller-namespace", metav1.NamespaceSystem, "Namespace of sealed-secrets controller.")
-	controllerName = flag.String("controller-name", "sealed-secrets-controller", "Name of sealed-secrets controller.")
-	outputFormat   = flag.StringP("format", "o", "json", "Output format for sealed secret. Either json or yaml")
-	outputFileName = flag.StringP("sealed-secret-file", "w", "", "Sealed-secret (output) file")
-	inputFileName  = flag.StringP("secret-file", "f", "", "Secret (input) file")
-	dumpCert       = flag.Bool("fetch-cert", false, "Write certificate to stdout. Useful for later use with --cert")
-	allowEmptyData = flag.Bool("allow-empty-data", false, "Allow empty data in the secret object")
-	printVersion   = flag.Bool("version", false, "Print version information and exit")
-	validateSecret = flag.Bool("validate", false, "Validate that the sealed secret can be decrypted")
-	mergeInto      = flag.String("merge-into", "", "Merge items from secret into an existing sealed secret file, updating the file in-place instead of writing to stdout.")
-	raw            = flag.Bool("raw", false, "Encrypt a raw value passed via the --from-* flags instead of the whole secret object")
-	secretName     = flag.String("name", "", "Name of the sealed secret (required with --raw and default (strict) scope)")
-	fromFile       = flag.StringSlice("from-file", nil, "(only with --raw) Secret items can be sourced from files. Pro-tip: you can use /dev/stdin to read pipe input. This flag tries to follow the same syntax as in kubectl")
-	sealingScope   ssv1alpha1.SealingScope
-	reEncrypt      bool // re-encrypt command
-	unseal         = flag.Bool("recovery-unseal", false, "Decrypt a sealed secrets file obtained from stdin, using the private key passed with --recovery-private-key. Intended to be used in disaster recovery mode.")
-	privKeys       = flag.StringSlice("recovery-private-key", nil, "Private key filename used by the --recovery-unseal command. Multiple files accepted either via comma separated list or by repetition of the flag. Either PEM encoded private keys or a backup of a json/yaml encoded k8s sealed-secret controller secret (and v1.List) are accepted. ")
-
 	// VERSION set from Makefile
 	VERSION = buildinfo.DefaultVersion
-
-	clientConfig clientcmd.ClientConfig
-
-	// testing hook for clientConfig.Namespace()
-	namespaceFromClientConfig = func() (string, bool, error) { return clientConfig.Namespace() }
 )
 
-func init() {
-	buildinfo.FallbackVersion(&VERSION, buildinfo.DefaultVersion)
+type namespaceFn func() (string, bool, error)
 
-	flag.Var(&sealingScope, "scope", "Set the scope of the sealed secret: strict, namespace-wide, cluster-wide (defaults to strict). Mandatory for --raw, otherwise the 'sealedsecrets.bitnami.com/cluster-wide' and 'sealedsecrets.bitnami.com/namespace-wide' annotations on the input secret can be used to select the scope.")
-	flag.BoolVar(&reEncrypt, "rotate", false, "")
-	flag.BoolVar(&reEncrypt, "re-encrypt", false, "Re-encrypt the given sealed secret to use the latest cluster key.")
+type Flags struct {
+	certURL        string
+	controllerNs   string
+	controllerName string
+	outputFormat   string
+	outputFileName string
+	inputFileName  string
+	kubeconfig     string
+	dumpCert       bool
+	allowEmptyData bool
+	printVersion   bool
+	validateSecret bool
+	mergeInto      string
+	raw            bool
+	secretName     string
+	fromFile       []string
+	sealingScope   ssv1alpha1.SealingScope
+	reEncrypt      bool
+	unseal         bool
+	privKeys       []string
+}
+
+type Config struct {
+	flags          *Flags
+	clientConfig   clientcmd.ClientConfig
+	ctx            context.Context
+	solveNamespace namespaceFn
+}
+
+func (f *Flags) Bind(fs *flag.FlagSet) {
+	if fs == nil {
+		fs = flag.CommandLine
+	}
+
+	// TODO: Verify k8s server signature against cert in kube client config.
+	fs.StringVar(&f.certURL, "cert", "", "Certificate / public key file/URL to use for encryption. Overrides --controller-*")
+	fs.StringVar(&f.controllerNs, "controller-namespace", metav1.NamespaceSystem, "Namespace of sealed-secrets controller.")
+	fs.StringVar(&f.controllerName, "controller-name", "sealed-secrets-controller", "Name of sealed-secrets controller.")
+	fs.StringVarP(&f.outputFormat, "format", "o", "json", "Output format for sealed secret. Either json or yaml")
+	fs.StringVarP(&f.outputFileName, "sealed-secret-file", "w", "", "Sealed-secret (output) file")
+	fs.StringVarP(&f.inputFileName, "secret-file", "f", "", "Secret (input) file")
+	fs.BoolVar(&f.dumpCert, "fetch-cert", false, "Write certificate to stdout. Useful for later use with --cert")
+	fs.BoolVar(&f.allowEmptyData, "allow-empty-data", false, "Allow empty data in the secret object")
+	fs.BoolVar(&f.printVersion, "version", false, "Print version information and exit")
+	fs.BoolVar(&f.validateSecret, "validate", false, "Validate that the sealed secret can be decrypted")
+	fs.StringVar(&f.mergeInto, "merge-into", "", "Merge items from secret into an existing sealed secret file, updating the file in-place instead of writing to stdout.")
+	fs.BoolVar(&f.raw, "raw", false, "Encrypt a raw value passed via the --from-* flags instead of the whole secret object")
+	fs.StringVar(&f.secretName, "name", "", "Name of the sealed secret (required with --raw and default (strict) scope)")
+	fs.StringSliceVar(&f.fromFile, "from-file", nil, "(only with --raw) Secret items can be sourced from files. Pro-tip: you can use /dev/stdin to read pipe input. This flag tries to follow the same syntax as in kubectl")
+	fs.StringVar(&f.kubeconfig, "kubeconfig", "", "Path to a kube config. Only required if out-of-cluster")
+
+	fs.Var(&f.sealingScope, "scope", "Set the scope of the sealed secret: strict, namespace-wide, cluster-wide (defaults to strict). Mandatory for --raw, otherwise the 'sealedsecrets.bitnami.com/cluster-wide' and 'sealedsecrets.bitnami.com/namespace-wide' annotations on the input secret can be used to select the scope.")
+	fs.BoolVar(&f.reEncrypt, "rotate", false, "")
+	fs.BoolVar(&f.reEncrypt, "re-encrypt", false, "Re-encrypt the given sealed secret to use the latest cluster key.")
 	_ = flag.CommandLine.MarkDeprecated("rotate", "please use --re-encrypt instead")
 
+	fs.BoolVar(&f.unseal, "recovery-unseal", false, "Decrypt a sealed secrets file obtained from stdin, using the private key passed with --recovery-private-key. Intended to be used in disaster recovery mode.")
+	fs.StringSliceVar(&f.privKeys, "recovery-private-key", nil, "Private key filename used by the --recovery-unseal command. Multiple files accepted either via comma separated list or by repetition of the flag. Either PEM encoded private keys or a backup of a json/yaml encoded k8s sealed-secret controller secret (and v1.List) are accepted. ")
+}
+
+func initUsualKubectlFlags(overrides *clientcmd.ConfigOverrides, flagset *flag.FlagSet) {
+	kflags := clientcmd.RecommendedConfigOverrideFlags("")
+	clientcmd.BindOverrideFlags(overrides, flagset, kflags)
+}
+
+func initClient(kubeConfigPath string, cfgOverrides *clientcmd.ConfigOverrides, r io.Reader) clientcmd.ClientConfig {
+	loadingRules := clientcmd.NewDefaultClientConfigLoadingRules()
+	loadingRules.DefaultClientConfig = &clientcmd.DefaultClientConfig
+	loadingRules.ExplicitPath = kubeConfigPath
+	return clientcmd.NewInteractiveDeferredLoadingClientConfig(loadingRules, cfgOverrides, r)
+}
+
+func initNamespaceFuncFromClient(clientConfig clientcmd.ClientConfig) namespaceFn {
+	return func() (string, bool, error) { return clientConfig.Namespace() }
+}
+
+func logFatal(err error) {
+	fmt.Fprintf(os.Stderr, "error: %v\n", err)
+	os.Exit(1)
+}
+
+func initConfigFromFlags() *Config {
+	var flags Flags
+	flags.Bind(nil)
+
+	buildinfo.FallbackVersion(&VERSION, buildinfo.DefaultVersion)
+
 	flagenv.SetFlagsFromEnv(flagEnvPrefix, goflag.CommandLine)
 
-	// The "usual" clientcmd/kubectl flags
-	loadingRules := clientcmd.NewDefaultClientConfigLoadingRules()
-	loadingRules.DefaultClientConfig = &clientcmd.DefaultClientConfig
-	overrides := clientcmd.ConfigOverrides{}
-	kflags := clientcmd.RecommendedConfigOverrideFlags("")
-	flag.StringVar(&loadingRules.ExplicitPath, "kubeconfig", "", "Path to a kube config. Only required if out-of-cluster")
-	clientcmd.BindOverrideFlags(&overrides, flag.CommandLine, kflags)
-	clientConfig = clientcmd.NewInteractiveDeferredLoadingClientConfig(loadingRules, &overrides, os.Stdin)
+	var overrides clientcmd.ConfigOverrides
+	initUsualKubectlFlags(&overrides, flag.CommandLine)
 
 	pflagenv.SetFlagsFromEnv(flagEnvPrefix, flag.CommandLine)
 
@@ -107,10 +156,24 @@ func init() {
 	klog.InitFlags(nil)
 	// Standard goflags (glog in particular)
 	flag.CommandLine.AddGoFlagSet(goflag.CommandLine)
+
+	flag.Parse()
+	if err := goflag.CommandLine.Parse([]string{}); err != nil {
+		logFatal(err)
+	}
+
+	clientConfig := initClient(flags.kubeconfig, &overrides, os.Stdin)
+	config := Config{
+		flags:          &flags,
+		clientConfig:   clientConfig,
+		ctx:            context.Background(),
+		solveNamespace: initNamespaceFuncFromClient(clientConfig),
+	}
+	return &config
 }
 
 func parseKey(r io.Reader) (*rsa.PublicKey, error) {
-	data, err := ioutil.ReadAll(r)
+	data, err := io.ReadAll(r)
 	if err != nil {
 		return nil, err
 	}
@@ -138,7 +201,7 @@ func parseKey(r io.Reader) (*rsa.PublicKey, error) {
 }
 
 func readSecret(codec runtime.Decoder, r io.Reader) (*v1.Secret, error) {
-	data, err := ioutil.ReadAll(r)
+	data, err := io.ReadAll(r)
 	if err != nil {
 		return nil, err
 	}
@@ -238,12 +301,12 @@ func openCertCluster(ctx context.Context, c corev1.CoreV1Interface, namespace, n
 	return cert, nil
 }
 
-func openCert(ctx context.Context, certURL string) (io.ReadCloser, error) {
+func openCert(cfg *Config, certURL string) (io.ReadCloser, error) {
 	if certURL != "" {
 		return openCertLocal(certURL)
 	}
 
-	conf, err := clientConfig.ClientConfig()
+	conf, err := cfg.clientConfig.ClientConfig()
 	if err != nil {
 		return nil, err
 	}
@@ -252,13 +315,13 @@ func openCert(ctx context.Context, certURL string) (io.ReadCloser, error) {
 	if err != nil {
 		return nil, err
 	}
-	return openCertCluster(ctx, restClient, *controllerNs, *controllerName)
+	return openCertCluster(cfg.ctx, restClient, cfg.flags.controllerNs, cfg.flags.controllerName)
 }
 
 // Seal reads a k8s Secret resource parsed from an input reader by a given codec, encrypts all its secrets
 // with a given public key, using the name and namespace found in the input secret, unless explicitly overridden
 // by the overrideName and overrideNamespace arguments.
-func seal(in io.Reader, out io.Writer, codecs runtimeserializer.CodecFactory, pubKey *rsa.PublicKey, scope ssv1alpha1.SealingScope, allowEmptyData bool, overrideName, overrideNamespace string) error {
+func seal(cfg *Config, in io.Reader, out io.Writer, codecs runtimeserializer.CodecFactory, pubKey *rsa.PublicKey, scope ssv1alpha1.SealingScope, allowEmptyData bool, overrideName, overrideNamespace string) error {
 	secret, err := readSecret(codecs.UniversalDecoder(), in)
 	if err != nil {
 		return err
@@ -285,7 +348,7 @@ func seal(in io.Reader, out io.Writer, codecs runtimeserializer.CodecFactory, pu
 	}
 
 	if ssv1alpha1.SecretScope(secret) != ssv1alpha1.ClusterWideScope && secret.GetNamespace() == "" {
-		ns, _, err := namespaceFromClientConfig()
+		ns, _, err := cfg.solveNamespace()
 		if clientcmd.IsEmptyConfig(err) {
 			return fmt.Errorf("input secret has no namespace and cannot infer the namespace automatically when no kube config is available")
 		} else if err != nil {
@@ -307,14 +370,15 @@ func seal(in io.Reader, out io.Writer, codecs runtimeserializer.CodecFactory, pu
 	if err != nil {
 		return err
 	}
-	if err = sealedSecretOutput(out, codecs, ssecret); err != nil {
+	if err = sealedSecretOutput(out, cfg.flags, codecs, ssecret); err != nil {
 		return err
 	}
 	return nil
 }
 
-func validateSealedSecret(ctx context.Context, in io.Reader, namespace, name string) error {
-	conf, err := clientConfig.ClientConfig()
+func validateSealedSecret(cfg *Config, in io.Reader) error {
+	flags := cfg.flags
+	conf, err := cfg.clientConfig.ClientConfig()
 	if err != nil {
 		return err
 	}
@@ -322,25 +386,25 @@ func validateSealedSecret(ctx context.Context, in io.Reader, namespace, name str
 	if err != nil {
 		return err
 	}
-	portName, err := getServicePortName(ctx, restClient, namespace, name)
+	portName, err := getServicePortName(cfg.ctx, restClient, flags.controllerNs, flags.controllerName)
 	if err != nil {
 		return err
 	}
 
-	content, err := ioutil.ReadAll(in)
+	content, err := io.ReadAll(in)
 	if err != nil {
 		return err
 	}
 
 	req := restClient.RESTClient().Post().
-		Namespace(namespace).
+		Namespace(flags.controllerNs).
 		Resource("services").
 		SubResource("proxy").
-		Name(net.JoinSchemeNamePort("http", name, portName)).
+		Name(net.JoinSchemeNamePort("http", flags.controllerName, portName)).
 		Suffix("/v1/verify")
 
 	req.Body(content)
-	res := req.Do(ctx)
+	res := req.Do(cfg.ctx)
 	if err := res.Error(); err != nil {
 		if status, ok := err.(*k8serrors.StatusError); ok && status.Status().Code == http.StatusConflict {
 			return fmt.Errorf("unable to decrypt sealed secret")
@@ -351,8 +415,9 @@ func validateSealedSecret(ctx context.Context, in io.Reader, namespace, name str
 	return nil
 }
 
-func reEncryptSealedSecret(ctx context.Context, in io.Reader, out io.Writer, codecs runtimeserializer.CodecFactory, namespace, name string) error {
-	conf, err := clientConfig.ClientConfig()
+func reEncryptSealedSecret(cfg *Config, in io.Reader, out io.Writer, codecs runtimeserializer.CodecFactory) error {
+	flags := cfg.flags
+	conf, err := cfg.clientConfig.ClientConfig()
 	if err != nil {
 		return err
 	}
@@ -360,25 +425,25 @@ func reEncryptSealedSecret(ctx context.Context, in io.Reader, out io.Writer, cod
 	if err != nil {
 		return err
 	}
-	portName, err := getServicePortName(ctx, restClient, namespace, name)
+	portName, err := getServicePortName(cfg.ctx, restClient, flags.controllerNs, flags.controllerName)
 	if err != nil {
 		return err
 	}
 
-	content, err := ioutil.ReadAll(in)
+	content, err := io.ReadAll(in)
 	if err != nil {
 		return err
 	}
 
 	req := restClient.RESTClient().Post().
-		Namespace(namespace).
+		Namespace(flags.controllerNs).
 		Resource("services").
 		SubResource("proxy").
-		Name(net.JoinSchemeNamePort("http", name, portName)).
+		Name(net.JoinSchemeNamePort("http", flags.controllerName, portName)).
 		Suffix("/v1/rotate")
 
 	req.Body(content)
-	res := req.Do(ctx)
+	res := req.Do(cfg.ctx)
 	if err := res.Error(); err != nil {
 		if status, ok := err.(*k8serrors.StatusError); ok && status.Status().Code == http.StatusConflict {
 			return fmt.Errorf("unable to rotate secret")
@@ -396,18 +461,21 @@ func reEncryptSealedSecret(ctx context.Context, in io.Reader, out io.Writer, cod
 	ssecret.SetCreationTimestamp(metav1.Time{})
 	ssecret.SetDeletionTimestamp(nil)
 	ssecret.Generation = 0
-	return sealedSecretOutput(out, codecs, ssecret)
+	if err = sealedSecretOutput(out, cfg.flags, codecs, ssecret); err != nil {
+		return err
+	}
+	return nil
 }
 
-func resourceOutput(out io.Writer, codecs runtimeserializer.CodecFactory, gv runtime.GroupVersioner, obj runtime.Object) error {
+func resourceOutput(out io.Writer, flags *Flags, codecs runtimeserializer.CodecFactory, gv runtime.GroupVersioner, obj runtime.Object) error {
 	var contentType string
-	switch strings.ToLower(*outputFormat) {
+	switch strings.ToLower(flags.outputFormat) {
 	case "json", "":
 		contentType = runtime.ContentTypeJSON
 	case "yaml":
 		contentType = runtime.ContentTypeYAML
 	default:
-		return fmt.Errorf("unsupported output format: %s", *outputFormat)
+		return fmt.Errorf("unsupported output format: %s", flags.outputFormat)
 	}
 	prettyEnc, err := prettyEncoder(codecs, contentType, gv)
 	if err != nil {
@@ -422,8 +490,8 @@ func resourceOutput(out io.Writer, codecs runtimeserializer.CodecFactory, gv run
 	return nil
 }
 
-func sealedSecretOutput(out io.Writer, codecs runtimeserializer.CodecFactory, ssecret *ssv1alpha1.SealedSecret) error {
-	return resourceOutput(out, codecs, ssv1alpha1.SchemeGroupVersion, ssecret)
+func sealedSecretOutput(out io.Writer, flags *Flags, codecs runtimeserializer.CodecFactory, ssecret *ssv1alpha1.SealedSecret) error {
+	return resourceOutput(out, flags, codecs, ssv1alpha1.SchemeGroupVersion, ssecret)
 }
 
 func decodeSealedSecret(codecs runtimeserializer.CodecFactory, b []byte) (*ssv1alpha1.SealedSecret, error) {
@@ -434,7 +502,7 @@ func decodeSealedSecret(codecs runtimeserializer.CodecFactory, b []byte) (*ssv1a
 	return &ss, nil
 }
 
-func sealMergingInto(in io.Reader, filename string, codecs runtimeserializer.CodecFactory, pubKey *rsa.PublicKey, scope ssv1alpha1.SealingScope, allowEmptyData bool) error {
+func sealMergingInto(cfg *Config, in io.Reader, filename string, codecs runtimeserializer.CodecFactory, pubKey *rsa.PublicKey, scope ssv1alpha1.SealingScope, allowEmptyData bool) error {
 	// #nosec G304 -- should open user provided file
 	f, err := os.OpenFile(filename, os.O_RDWR, 0)
 	if err != nil {
@@ -454,7 +522,7 @@ func sealMergingInto(in io.Reader, filename string, codecs runtimeserializer.Cod
 	}
 
 	var buf bytes.Buffer
-	if err := seal(in, &buf, codecs, pubKey, scope, allowEmptyData, orig.Name, orig.Namespace); err != nil {
+	if err := seal(cfg, in, &buf, codecs, pubKey, scope, allowEmptyData, orig.Name, orig.Namespace); err != nil {
 		return err
 	}
 
@@ -479,7 +547,7 @@ func sealMergingInto(in io.Reader, filename string, codecs runtimeserializer.Cod
 
 	// updated sealed secret file in-place avoiding clobbering the file upon rendering errors.
 	var out bytes.Buffer
-	if err := sealedSecretOutput(&out, codecs, orig); err != nil {
+	if err := sealedSecretOutput(&out, cfg.flags, codecs, orig); err != nil {
 		return err
 	}
 
@@ -523,7 +591,7 @@ func parseFromFile(s string) (string, string) {
 
 func readPrivKeysFromFile(filename string) ([]*rsa.PrivateKey, error) {
 	// #nosec G304 -- should open user provided file
-	b, err := ioutil.ReadFile(filename)
+	b, err := os.ReadFile(filename)
 	if err != nil {
 		return nil, err
 	}
@@ -610,13 +678,13 @@ func readPrivKeys(filenames []string) (map[string]*rsa.PrivateKey, error) {
 	return res, nil
 }
 
-func unsealSealedSecret(w io.Writer, in io.Reader, codecs runtimeserializer.CodecFactory, privKeyFilenames []string) error {
-	privKeys, err := readPrivKeys(privKeyFilenames)
+func unsealSealedSecret(flags *Flags, w io.Writer, in io.Reader, codecs runtimeserializer.CodecFactory) error {
+	privKeys, err := readPrivKeys(flags.privKeys)
 	if err != nil {
 		return err
 	}
 
-	b, err := ioutil.ReadAll(in)
+	b, err := io.ReadAll(in)
 	if err != nil {
 		return err
 	}
@@ -630,23 +698,24 @@ func unsealSealedSecret(w io.Writer, in io.Reader, codecs runtimeserializer.Code
 		return err
 	}
 
-	return resourceOutput(w, codecs, v1.SchemeGroupVersion, sec)
+	return resourceOutput(w, flags, codecs, v1.SchemeGroupVersion, sec)
 }
 
-func run(ctx context.Context, w io.Writer, inputFileName, outputFileName, secretName, controllerNs, controllerName, certURL string, printVersion, validateSecret, reEncrypt, dumpCert, raw, allowEmptyData bool, fromFile []string, mergeInto string, unseal bool, privKeys []string) (err error) {
-	if len(fromFile) != 0 && !raw {
+func run(w io.Writer, cfg *Config) (err error) {
+	flags := cfg.flags
+	if len(flags.fromFile) != 0 && !flags.raw {
 		return fmt.Errorf("--from-file requires --raw")
 	}
 
-	if printVersion {
+	if flags.printVersion {
 		fmt.Fprintf(w, "kubeseal version: %s\n", VERSION)
 		return nil
 	}
 
 	var input io.Reader = os.Stdin
-	if inputFileName != "" {
+	if flags.inputFileName != "" {
 		// #nosec G304 -- should open user provided file
-		f, err := os.Open(inputFileName)
+		f, err := os.Open(flags.inputFileName)
 		if err != nil {
 			return nil
 		}
@@ -654,7 +723,7 @@ func run(ctx context.Context, w io.Writer, inputFileName, outputFileName, secret
 		defer f.Close()
 
 		input = f
-	} else if !raw && !dumpCert {
+	} else if !flags.raw && !flags.dumpCert {
 		if isatty.IsTerminal(os.Stdin.Fd()) {
 			fmt.Fprintf(os.Stderr, "(tty detected: expecting json/yaml k8s resource in stdin)\n")
 		}
@@ -662,17 +731,16 @@ func run(ctx context.Context, w io.Writer, inputFileName, outputFileName, secret
 
 	// reEncrypt is the only "in-place" update subcommand. When the user only provides one file (the input file)
 	// we'll use the same file for output (see #405).
-	if reEncrypt && (outputFileName == "" && inputFileName != "") {
-		outputFileName = inputFileName
+	if flags.reEncrypt && (flags.outputFileName == "" && flags.inputFileName != "") {
+		flags.outputFileName = flags.inputFileName
 	}
-	if outputFileName != "" {
-		// TODO(mkm): get rid of these horrible global variables
-		if ext := filepath.Ext(outputFileName); ext == ".yaml" || ext == ".yml" {
-			*outputFormat = "yaml"
+	if flags.outputFileName != "" {
+		if ext := filepath.Ext(flags.outputFileName); ext == ".yaml" || ext == ".yml" {
+			flags.outputFormat = "yaml"
 		}
 
 		var f *renameio.PendingFile
-		f, err = renameio.TempFile("", outputFileName)
+		f, err = renameio.TempFile("", flags.outputFileName)
 		if err != nil {
 			return err
 		}
@@ -686,28 +754,28 @@ func run(ctx context.Context, w io.Writer, inputFileName, outputFileName, secret
 		w = f
 	}
 
-	if unseal {
-		return unsealSealedSecret(w, input, scheme.Codecs, privKeys)
+	if flags.unseal {
+		return unsealSealedSecret(flags, w, input, scheme.Codecs)
 	}
-	if len(privKeys) != 0 && isatty.IsTerminal(os.Stderr.Fd()) {
+	if len(flags.privKeys) != 0 && isatty.IsTerminal(os.Stderr.Fd()) {
 		fmt.Fprintf(os.Stderr, "warning: ignoring --recovery-private-key because unseal command not chosen with --recovery-unseal\n")
 	}
 
-	if validateSecret {
-		return validateSealedSecret(ctx, input, controllerNs, controllerName)
+	if flags.validateSecret {
+		return validateSealedSecret(cfg, input)
 	}
 
-	if reEncrypt {
-		return reEncryptSealedSecret(ctx, input, w, scheme.Codecs, controllerNs, controllerName)
+	if flags.reEncrypt {
+		return reEncryptSealedSecret(cfg, input, w, scheme.Codecs)
 	}
 
-	f, err := openCert(ctx, certURL)
+	f, err := openCert(cfg, flags.certURL)
 	if err != nil {
 		return err
 	}
 	defer f.Close()
 
-	if dumpCert {
+	if flags.dumpCert {
 		_, err := io.Copy(w, f)
 		return err
 	}
@@ -717,61 +785,58 @@ func run(ctx context.Context, w io.Writer, inputFileName, outputFileName, secret
 		return err
 	}
 
-	if mergeInto != "" {
-		return sealMergingInto(input, mergeInto, scheme.Codecs, pubKey, sealingScope, allowEmptyData)
+	if flags.mergeInto != "" {
+		return sealMergingInto(cfg, input, flags.mergeInto, scheme.Codecs, pubKey, flags.sealingScope, flags.allowEmptyData)
 	}
 
-	if raw {
+	if flags.raw {
 		var (
 			ns  string
 			err error
 		)
-		if sealingScope < ssv1alpha1.ClusterWideScope {
-			ns, _, err = namespaceFromClientConfig()
+		if flags.sealingScope < ssv1alpha1.ClusterWideScope {
+			ns, _, err = cfg.solveNamespace()
 			if err != nil {
 				return err
 			}
 
 			if ns == "" {
-				return fmt.Errorf("must provide the --namespace flag with --raw and --scope %s", sealingScope.String())
+				return fmt.Errorf("must provide the --namespace flag with --raw and --scope %s", flags.sealingScope.String())
 			}
 
-			if secretName == "" && sealingScope < ssv1alpha1.NamespaceWideScope {
-				return fmt.Errorf("must provide the --name flag with --raw and --scope %s", sealingScope.String())
+			if flags.secretName == "" && flags.sealingScope < ssv1alpha1.NamespaceWideScope {
+				return fmt.Errorf("must provide the --name flag with --raw and --scope %s", flags.sealingScope.String())
 			}
 		}
 
 		var data []byte
-		if len(fromFile) > 0 {
-			if len(fromFile) > 1 {
+		if len(flags.fromFile) > 0 {
+			if len(flags.fromFile) > 1 {
 				return fmt.Errorf("must provide only one --from-file when encrypting a single item with --raw")
 			}
 
-			_, filename := parseFromFile(fromFile[0])
+			_, filename := parseFromFile(flags.fromFile[0])
 			// #nosec G304 -- should open user provided file
-			data, err = ioutil.ReadFile(filename)
+			data, err = os.ReadFile(filename)
 		} else {
 			if isatty.IsTerminal(os.Stdin.Fd()) {
 				fmt.Fprintf(os.Stderr, "(tty detected: expecting a secret to encrypt in stdin)\n")
 			}
-			data, err = ioutil.ReadAll(os.Stdin)
+			data, err = io.ReadAll(os.Stdin)
 		}
 		if err != nil {
 			return err
 		}
 
-		return encryptSecretItem(w, secretName, ns, data, sealingScope, pubKey)
+		return encryptSecretItem(w, flags.secretName, ns, data, flags.sealingScope, pubKey)
 	}
 
-	return seal(input, w, scheme.Codecs, pubKey, sealingScope, allowEmptyData, secretName, "")
+	return seal(cfg, input, w, scheme.Codecs, pubKey, flags.sealingScope, flags.allowEmptyData, flags.secretName, "")
 }
 
 func main() {
-	flag.Parse()
-	_ = goflag.CommandLine.Parse([]string{})
-
-	if err := run(context.Background(), os.Stdout, *inputFileName, *outputFileName, *secretName, *controllerNs, *controllerName, *certURL, *printVersion, *validateSecret, reEncrypt, *dumpCert, *raw, *allowEmptyData, *fromFile, *mergeInto, *unseal, *privKeys); err != nil {
-		fmt.Fprintf(os.Stderr, "error: %v\n", err)
-		os.Exit(1)
+	cfg := initConfigFromFlags()
+	if err := run(os.Stdout, cfg); err != nil {
+		logFatal(err)
 	}
 }
